
local AbsorbsMonitor = _G.AbsorbsMonitor;

local UnitStats;

local Scaling;
local playerScaling;
local privateScaling;

local playerGUID;
local playerClass;


-- #################################################################################################


-- No function in this file will be called before this
function AbsorbsMonitor:EnableEffects()
	UnitStats = AbsorbsMonitor.UnitStats;

	Scaling = AbsorbsMonitor.Scaling;
	playerGUID = UnitGUID("player");
	_, playerClass = UnitClass("player");
	
	playerScaling = Scaling[playerGUID];
	privateScaling = Scaling[-1];
end


-- #################################################################################################


-- Public Scaling: { [MagicSuppression] }

local deathknight_MS_Ranks = {[0] = 0, [1] = 0.08, [2] = 0.16, [3] = 0.25};

function deathknight_AntiMagicShell_Create(sourceGUID, sourceName, destGUID, destName, spellId, destEffects)
	local maxHealth = UnitHealthMax(destName);
	
	if(maxHealth) then		
		return math.floor(maxHealth * 0.5), (Scaling[sourceGUID] and Scaling[sourceGUID][1]) and 1.0 or 0.5;
	else
		return 0, 0.0;
	end
end

local function deathknight_AntiMagicShell_Hit(guid, curEffect, absorbedRemaining, overkill, spellSchool) 
	-- TODO: what happens to mixed school attacks?
	if(spellSchool == SCHOOL_MASK_PHYSICAL) then 
		return 0, true;
	end
	
	-- TODO: make it possible to store custom data with an active absorb effect
	local maxAbsorb = (absorbedRemaining + overkill) * (0.75 + (Scaling[guid] and Scaling[guid][1] or 0));
	
	if(curEffect[3] < maxAbsorb) then
		return curEffect[3], false;
	else
		return maxAbsorb, true;
	end
end

local function deathknight_OnTalentUpdate(self)
	local _, _, _, _, t = GetTalentInfo(3, 18);
	
	playerScaling[1] = deathknight_MS_Ranks[t];
end

function AbsorbsMonitor:OnEnable_DEATHKNIGHT()
	self.OnTalentUpdate = deathknight_OnTalentUpdate;
		
	self:ScheduleTimer(deathknight_OnTalentUpdate, 2.0, self);
end


-- #################################################################################################


local function druid_SavageDefense_Create(sourceGUID, sourceName, destGUID, destName, spellId, destEffects)
	if(sourceGUID == playerGUID) then
		local baseAP, plusAP, minusAP = UnitAttackPower("player");
	
		return math.floor((baseAP + plusAP - minusAP) * 0.25), 1.0;
	else
		return 0, 0.0
	end
end

local function druid_SavageDefense_Hit(guid, curEffect, absorbedRemaining, overkill, spellSchool) 
	-- TODO: what happens to mixed school attacks?
	if(spellSchool == SCHOOL_MASK_PHYSICAL) then 
		return min(curEffect[3], absorbedRemaining), false;
	else
		return 0, true;
	end
	
	
end


-- #################################################################################################


-- Table for base values of
-- Fire Ward, Frost Ward, Ice Barrier, Mana Shield
local mage_Absorb_Spells = {
	-- Fire Ward
	[543] = 165,
	[8457] = 290,
	[8458] = 470,
	[10223] = 675,
	[10225] = 875,
	[27218] = 1125,
	[43010] = 1950,
	
	-- Frost Ward
	[6143] = 165,
	[8461] = 290,
	[8462] = 470,
	[10177] = 675,
	[28609] = 875,
	[32796] = 1125,
	[43012] = 1950,
	
	-- Ice Barrier
	[11426] = 438,
	[13031] = 549,
	[13032] = 678,
	[13033] = 818,
	[27134] = 925,
	[33405] = 1075,
	[43038] = 2800,
	[43039] = 3300,
	
	-- Mana Shield
	[1463] = 120,
	[8494] = 210,
	[8495] = 300,
	[10191] = 390,
	[10192] = 480,
	[10193] = 570,
	[27131] = 715,
	[43019] = 1080,
	[43020] = 1330,
};


-- No Downranking support here
-- Ice Barrier also increases in base value when you level, but there's a new
-- rank at 80. Since all these level scaling effects get revamped completely in Cataclysm,
-- I will add no support here
local function mage_IceBarrier_Create(sourceGUID, sourceName, destGUID, destName, spellId, destEffects)
	if(sourceGUID ~= playerGUID) then return 0, 0.0; end
	
	local base = (mage_Absorb_Spells[spellId] + (GetSpellBonusHealing() * 0.8053));
	
	if(Scaling[sourceGUID] and Scaling[sourceGUID][1]) then
		return math.floor(base * Scaling[sourceGUID][1]), 1.0;
	else
		return math.floor(base), 0.4;
	end
	
	
end

local function mage_FireWard_Hit(guid, curEffect, absorbedRemaining, overkill, spellSchool)
	if(spellSchool ~= SCHOOL_MASK_FIRE) then
		return 0, true;
	end
	
	return generic_Hit(guid, curEffect, absorbedRemaining, overkill, spellSchool);
end

local function mage_FrostWard_Hit(guid, curEffect, absorbedRemaining, overkill, spellSchool)
	if(spellSchool ~= SCHOOL_MASK_FROST) then
		return 0, true;
	end
	
	return generic_Hit(guid, curEffect, absorbedRemaining, overkill, spellSchool);
end

local function mage_OnGlyphUpdated(self)
	local glyphSpellId;
	
	playerScaling[1] = 1.0;

	for i = 1, 3 do
		_, _glyphSpellId = GetGlyphSocketInfo(i);
		
		if(glyphSpellId == 63095) then
			playerScaling[1] = 1.3;
			
			break;
		end
	 end		
end

function AbsorbsMonitor:OnEnable_MAGE()
	self.OnGlyphUpdated = mage_OnGlyphUpdated;
	
	self:ScheduleTimer(mage_OnGlyphUpdated, 2.0, self);
end


-- #################################################################################################


local function paladin_SacredShield_Create(sourceGUID, sourceName, destGUID, destName, spellId, destEffects)
	return 0, 0.0;
end


-- #################################################################################################


PRIEST_DIVINEAEGIS_SPELLID = 47753;

-- [rank] = {spellId, level, value}
-- Value will be extracted from the spell book by priest_ScanSpellbook()
local priest_PWS_Ranks = {
	[1] = {17, 6, 0},
	[2] = {592, 12, 0},
	[3] = {600, 18, 0},
	[4] = {3747, 24, 0},
	[5] = {6065, 30, 0},
	[6] = {6066, 36, 0},
	[7] = {10898, 42, 0},
	[8] = {10899, 48, 0},
	[9] = {10900, 54, 0},
	[10] = {10901, 60, 0},
	[11] = {25217, 65, 0},
	[12] = {25218, 70, 0},
	[13] = {48065, 75, 0},
	[14] = {48066, 80, 0}
};

-- Public Scaling:
--   Power Word: Shield: [spellId] = {base, spFactor}
--   Divine Aegis: [47753] = healFactor

-- Private Scaling
--   Talents: "TwinDisc", "ImpPWS", "FocusedPower", "DivineAegis", "BorrowedTime", "SpiritualHealing"
--	 Gear: "4pcRaid9", "4pcRaid10"

local function priest_PowerWordShield_Create(sourceGUID, sourceName, destGUID, destName, spellId, destEffects)
	local value;
	
	if(sourceGUID == playerGUID) then
		return math.floor((playerScaling[spellId][1] + GetSpellBonusHealing() * playerScaling[spellId][2]) * AbsorbsMonitor.db.profile.zoneModifier), 1.0;
	else
		return 0, 0.0;
	end
end

local function priest_DivineAegis_Create(sourceGUID, sourceName, destGUID, destName, spellId, destEffects)		
	local existing = 0;
	
	if(destEffects and destEffects[spellId]) then
		existing = destEffects[spellId][3];
	end

	local charged = AbsorbsMonitor:ConsumeCharges(destGUID, spellId);
	
	if(charged == 0) then
		return existing, 0.0;
	end
	
	local destLevel = UnitLevel(destName);
	local quality = 1.0;		
	
	if(not destLevel) then
		destLevel = 80;
		quality = 0.4;
	end
		
	return min(destLevel * 125, existing + charged), quality;
end

-- I officially HATE Divine Aegis (and Val'anyr for that matter) from now. 
-- The latency is horrible sometimes, even 5.0 is too low on Holy Nova spam. I suspect
-- that combat log order issues lead to expired charges, because two critical heals occur and THEN
-- two Divine Aegis refreshments arrive. In that case, putting single charge would solve the problem,
-- but I think it doesn't matter - since any absorb in the meantime will have screwed our calculations
-- anyway
local function priest_DivineAegis_OnHealCrit(sourceGUID, sourceName, destGUID, destName, spellId, amount)	
	-- No need for any check anymore, since the callback is only in place
	-- if the source has Divine Aegis	
	
	AbsorbsMonitor:AddCharge(destGUID, PRIEST_DIVINEAEGIS_SPELLID, math.floor(amount * playerScaling[PRIEST_DIVINEAEGIS_SPELLID]), 5.0);
end

local function priest_CalculateScaling()
	local playerLevel_minus5 = UnitLevel("player") - 5;
	local base_spFactor = 0.807;
	
	base_spFactor = base_spFactor + (privateScaling["BorrowedTime"] * 0.08);
	base_spFactor = base_spFactor * (1.0 + (privateScaling["TwinDisc"] * 0.01) + (privateScaling["FocusedPower"] * 0.02) + (privateScaling["SpiritualHealing"] * 0.02)) * (1.0 + ((privateScaling["ImpPWS"] + privateScaling["4pcRaid10"]) * 0.05));	
	
	playerScaling[PRIEST_DIVINEAEGIS_SPELLID] = (privateScaling["DivineAegis"] * 0.1) * (1 + (privateScaling["4pcRaid9"] * 0.03));
	
	if(playerScaling[PRIEST_DIVINEAEGIS_SPELLID] > 0) then
		AbsorbsMonitor:AddCustomTrigger(playerGUID, "OnHealCrit", priest_DivineAegis_OnHealCrit);
	else
		AbsorbsMonitor:RemoveCustomTrigger(playerGUID, "OnHealCrit", priest_DivineAegis_OnHealCrit);
	end
	
	local spFactor;
	
	for k, v in pairs(priest_PWS_Ranks) do
		if(v[3]) then
			-- Based on the assumption that the decrease in sp coefficient is linear,
			-- only tested for level 80 though so far
			-- Cataclysm will help us get rid of this crap again
			if(v[2] < playerLevel_minus5) then
				if(playerLevel_minus5 == 75) then
					spFactor = max(base_spFactor * ((v[2] * 0.045228921) - 2.381389768), 0);
				else
					spFactor = 0;
				end
			else
				spFactor = base_spFactor;
			end
			
			playerScaling[v[1]] = {v[3] * (1.0 + (privateScaling["TwinDisc"] * 0.01) + (privateScaling["FocusedPower"] * 0.02) + (privateScaling["SpiritualHealing"] * 0.02)), spFactor};
		end
	end
end

-- Because Power Word: Shield scales very slightly with level, we have
-- to scan the spellbook sadly...
local function priest_ScanSpellbook()
	local PWS_Tooltip = {};
	PWS_Tooltip.t = CreateFrame("GameTooltip");
	PWS_Tooltip.t:SetOwner(UIParent, "ANCHOR_NONE");
	
	for i = 1, 4 do
		PWS_Tooltip["TooltipTextLeft" .. i] = PWS_Tooltip.t:CreateFontString();
		PWS_Tooltip["TooltipTextRight" .. i] = PWS_Tooltip.t:CreateFontString();
		PWS_Tooltip.t:AddFontStrings(PWS_Tooltip["TooltipTextLeft" .. i], PWS_Tooltip["TooltipTextRight" .. i]);
	end
		
	local spellName, spellRank, spellId;
	local i = 1;
	
	while true do
		spellName, spellRank = GetSpellName(i, BOOKTYPE_SPELL);
		
		if(not spellName) then break; end
		
		if(spellName == "Power Word: Shield") then
			spellRank = tonumber(spellRank:match("(%d+)"));			
			spellId = priest_PWS_Ranks[spellRank][1];
			
			if(not spellId) then
				self:Print("could not found spell id for rank "..spellRank);
				break;
			end						
			
			PWS_Tooltip.t:SetSpell(i, BOOKTYPE_SPELL);
			
			local _, _, value = string.find(PWS_Tooltip.TooltipTextLeft4:GetText(), "absorbing (%d+) damage");						
			
			priest_PWS_Ranks[spellRank][3] = value;			
		end
		
		i = i + 1;
	end	
	
	PWS_Tooltip.t:SetParent(nil);
	PWS_Tooltip = nil;
end

local function priest_ScanTalents()
	-- Twin Disciplines
	local _, _, _, _, t = GetTalentInfo(1, 2);
	privateScaling["TwinDisc"] = t;
	
	-- Improved Power Word: Shield
	local _, _, _, _, t = GetTalentInfo(1, 7);
	privateScaling["ImpPWS"] = t;
	
	-- Focused Power
	local _, _, _, _, t = GetTalentInfo(1, 16);
	privateScaling["FocusedPower"] = t;
	
	-- Divine Aegis
	local _, _, _, _, t = GetTalentInfo(1, 24);		
	privateScaling["DivineAegis"] = t;
	
	-- Borrowed Time
	local _, _, _, _, t = GetTalentInfo(1, 27);
	privateScaling["BorrowedTime"] = t;
	
	-- Spiritual Healing
	local _, _, _, _, t = GetTalentInfo(2, 16);
	privateScaling["SpiritualHealing"] = t;
end

local function priest_ScanEquipment()
	local n = 0;
	
	-- Crimson Acolyte Raiment's 4-piece Bonus
	if(IsEquippedItem("Crimson Acolyte Hood") or IsEquippedItem("Sanctified Crimson Acolyte Hood")) 				then n = 1; end
	if(IsEquippedItem("Crimson Acolyte Shoulderpads") or IsEquippedItem("Sanctified Crimson Acolyte Shoulderpads")) then n = n + 1; end
	if(IsEquippedItem("Crimson Acolyte Robe") or IsEquippedItem("Sanctified Crimson Acolyte Robe")) 				then n = n + 1; end
	if(IsEquippedItem("Crimson Acolyte Gloves") or IsEquippedItem("Sanctified Crimson Acolyte Gloves")) 			then n = n + 1; end
	if(IsEquippedItem("Crimson Acolyte Leggings") or IsEquippedItem("Sanctified Crimson Acolyte Leggings")) 		then n = n + 1; end
	
	if(n >= 4) then
		privateScaling["4pcRaid10"] = 1;
		
		 -- no way to have 4pcRaid9 now
		privateScaling["4pcRaid9"] = 0;		
		return;
	else
		privateScaling["4pcRaid10"] = 0;
	end
	
	n = 0;
	
	if(UnitFactionGroup("player") == "Alliance") then
		if(IsEquippedItem("Velen's Cowl of Conquest") or IsEquippedItem("Velen's Cowl of Triumph")) 				then n = 1; end
		if(IsEquippedItem("Velen's Shoulderpads of Conquest") or IsEquippedItem("Velen's Shoulderpads of Triumph")) then n = n + 1; end
		if(IsEquippedItem("Velen's Robe of Conquest") or IsEquippedItem("Velen's Robe of Triumph")) 				then n = n + 1; end
		if(IsEquippedItem("Velen's Gloves of Conquest") or IsEquippedItem("Velen's Gloves of Triumph")) 			then n = n + 1; end
		if(IsEquippedItem("Velen's Leggings of Conquest") or IsEquippedItem("Velen's Leggings of Triumph")) 		then n = n + 1; end
	else
		if(IsEquippedItem("Zabra's Cowl of Conquest") or IsEquippedItem("Zabra's Cowl of Triumph")) 				then n = 1; end
		if(IsEquippedItem("Zabra's Shoulderpads of Conquest") or IsEquippedItem("Zabra's Shoulderpads of Triumph")) then n = n + 1; end
		if(IsEquippedItem("Zabra's Robe of Conquest") or IsEquippedItem("Zabra's Robe of Triumph")) 				then n = n + 1; end
		if(IsEquippedItem("Zabra's Gloves of Conquest") or IsEquippedItem("Zabra's Gloves of Triumph")) 			then n = n + 1; end
		if(IsEquippedItem("Zabra's Leggings of Conquest") or IsEquippedItem("Zabra's Leggings of Triumph")) 		then n = n + 1; end	
	end
	
	if(n >= 4) then
		privateScaling["4pcRaid9"] = 1;
	else
		privateScaling["4pcRaid9"] = 0;
	end
end

local function priest_OnLevelUp(self)
	priest_ScanSpellbook();
	priest_CalculateScaling();
end

local function priest_OnTalentUpdate(self)
	priest_ScanTalents();
	priest_CalculateScaling();
end

local function priest_OnEquipmentChangedDelayed(self)
	priest_ScanEquipment();	
	priest_CalculateScaling();
	
	self:FinishUniqueTimer("OnEquipChanged");
end

local function priest_OnEquipmentChanged(self)
	self:ScheduleUniqueTimer("OnEquipChanged", priest_OnEquipmentChangedDelayed, 0.7, self);
end

function AbsorbsMonitor:OnEnable_PRIEST()
	self.OnLevelUp = priest_OnLevelUp;
	self.OnTalentUpdate = priest_OnTalentUpdate;
	self.OnEquipmentChanged = priest_OnEquipmentChanged;
	
	self:ScheduleTimer(
		function()
			priest_ScanSpellbook();
			priest_ScanTalents();
			priest_ScanEquipment();
		
			priest_CalculateScaling();
		end, 2.0);
end

-- #################################################################################################


local warlock_Sacrifice_Spells = {
	[7812] = 305,
	[19438] = 510,
	[19440] = 770,
	[19441] = 1095,
	[19442] = 1470,
	[19443] = 1905,
	[27273] = 2855,
	[47985] = 6750,
	--[47986] = 8350,
	-- There is a new rank of Sacrifice at 79, so it scales by 15 to 80
	[47986] = 8365,
}

local warlock_ShadowWard_Spells = {
	[6229] = 290,
	[11739] = 470,
	[11740] = 675,
	[28610] = 875,
	[47890] = 2750,
	[47891] = 3300,
};

-- Public Scaling: { [DemonicBrutality] }


local function warlock_Sacrifice_Create(sourceGUID, sourceName, destGUID, destName, spellId, destEffects)
	-- Note that the source is always the voidwalker!	
	if(Scaling[destGUID] and Scaling[destGUID][1]) then
		return math.floor(warlock_Sacrifice_Spells[spellId] * Scaling[destGUID][1]), 1.0;
	else
		return warlock_Sacrifice_Spells[spellId], 0.4;
	end
end

local function warlock_ShadowWard_Hit(guid, curEffect, absorbedRemaining, overkill, spellSchool)
	if(spellSchool ~= SCHOOL_MASK_SHADOW) then
		return 0, true;
	end
	
	return generic_Hit(guid, curEffect, absorbedRemaining, overkill, spellSchool);
end

local function warlock_OnTalentUpdate(self)
	local _, _, _, _, t = GetTalentInfo(2, 6);
	
	playerScaling[1] = t * 0.1;
end

function AbsorbsMonitor:OnEnable_WARLOCK()
	self.OnTalentUpdate = warlock_OnTalentUpdate;
	
	self:ScheduleTimer(warlock_OnTalentUpdate, 2.0, self);
end


-- #################################################################################################


local function items_EssenceOfGossamer_Hit(guid, curEffect)
	if(curEffect[3] < 140) then
		return curEffect[3], false;
	else
		return 140, true;
	end
end

local function items_Valanyr_OnAuraApplied(sourceGUID, sourceName, destGUID, destName, spellId)
	AbsorbsMonitor:AddCustomTrigger(sourceGUID, "OnHeal", items_Valanyr_OnHeal);
end

local function items_Valanyr_OnAuraRemoved(sourceGUID, sourceName, destGUID, destName, spellId)
	AbsorbsMonitor:RemoveCustomTrigger(sourceGUID, "OnHeal", items_Valanyr_OnHeal);
end

local function items_Valanyr_Create(sourceGUID, sourceName, destGUID, destName, spellId, destEffects)
	local existing = 0;
	
	if(destEffects and destEffects[spellId]) then
		existing = destEffects[spellId][3];
	end

	local charged = AbsorbsMonitor:ConsumeCharges(destGUID, spellId);
	
	if(charged == 0) then
		return existing, 0.0;
	end
		
	return min(20000, existing + charged), 1.0;
end

local function items_Valanyr_OnHeal(sourceGUID, sourceName, destGUID, destName, spellId, amount)
	AbsorbsMonitor:AddCharge(destGUID, 64413, math.floor(amount * 0.15), 5.0);
end


-- #################################################################################################


--- Generic Create function (only for documentary purposes)
-- @param	sourceGUID			guid of the originating unit
-- @param	sourceName			name of the originating unit
-- @param	destGUID			guid of the affected unit
-- @param	destName			name of the affected unit
-- @param	spellId				the spellId of this absorb effect
-- @param	destEffects			activeEffectsBySpell[destGUID]
-- @return	total value of this absorb effect
-- @return	quality (that is accuracy) of this value
local function generic_Create(sourceGUID, sourceName, destGUID, destName, spellId, destEffects)
end

local function generic_ConstantByTable_Create(sourceGUID, sourceName, destGUID, destName, spellId, destEffects)
	return AbsorbsMonitor.Effects[spellId][4][spellId], 1.0;
end

-- Generic Create function for spells simply scaling with spellpower and a fixed
-- coefficient.
-- Expects at effect[4] a table indexed by spellId with the base values and at
-- effect[5] the spellpower coefficient
local function generic_SpellScalingByTable_Create(sourceGUID, sourceName, destGUID, destName, spellId, destEffects)
	if(sourceGUID ~= playerGUID) then return 0, 0.0; end
	
	local effect = AbsorbsMonitor.Effects[spellId];

	return math.floor(effect[4][spellId] + (GetSpellBonusHealing() * effect[5])), 1.0;
end


--- Generic Hit function suitable for most absorb effects
-- Note that this function is only responsible for determining the amount this
-- particular absorb effect will take, NOT to handle its consequences like updating
-- the data structures
-- @param	guid				guid of affected unit
-- @param	curEffect			the value of activeEffectsBySpell[guid][spellId]
-- @param	absorbedRemaining	absorb value left to be accounted for on this unit
-- @param	overkill			amount of damage done on top of the absorb
-- @param	spellSchool			spell school for this hit
-- @return	absorb value this effect can account for
-- @return	whether this absorb was broken by this hit
local function generic_Hit(guid, curEffect, absorbedRemaining, overkill, spellSchool)
	if(absorbedRemaining > curEffect[3]) then
		-- dirty but efficient
		absorbedRemaining = curEffect[3];
		overkill = 1;
	end

	return absorbedRemaining, (overkill == 0);
end


-- #################################################################################################


local mage_FireWard_Entry = {2.0, generic_SpellScalingByTable_Create, mage_FireWard_Hit, mage_Absorb_Spells, 0.8053};
local mage_FrostWard_Entry = {2.0, generic_SpellScalingByTable_Create, mage_FrostWard_Hit, mage_Absorb_Spells, 0.8053};
local mage_IceBarrier_Entry = {1.0, mage_IceBarrier_Create, generic_Hit};
local mage_ManaShield_Entry = {1.0, mage_SpellScalingByTable_Create, generic_Hit, mage_Absorb_Spells, 0.8053};

local priest_PWS_Entry = {1.0, priest_PowerWordShield_Create, generic_Hit};	

local warlock_Sacrifice_Entry = {1.0, generic_ConstantByTable_Create, generic_Hit, warlock_Sacrifice_Spells};
local warlock_ShadowWard_Entry = {2.0, generic_SpellScalingByTable_Create, warlock_ShadowWard_Hit, warlock_ShadowWard_Spells, 0.8053};

local unsupportedEntry = {1, function() return 0, 0.0; end, generic_Hit};


-- INCOMPLETE
-- This table gets a lot smaller in Cataclysm
AbsorbsMonitor.Effects = {
	-- DEATH KNIGHT	
	-- Anti-Magic Shell
	[48707] = {3.0, deathknight_AntiMagicShell_Create, deathknight_AntiMagicShell_Hit},
		
	-- DRUID
	-- Savage Defense
	[62606] = {1.1, druid_SavageDefense_Create, druid_SavageDefense_Hit},
	-- Sacred Shield
	[58597] = {1.0, paladin_SacredShield_Create, generic_Hit},
	
	-- MAGE
	-- Fire Ward
	[543] = mage_FireWard_Entry,
	[8457] = mage_FireWard_Entry,
	[8458] = mage_FireWard_Entry,
	[10223] = mage_FireWard_Entry,
	[10225] = mage_FireWard_Entry,
	[27218] = mage_FireWard_Entry,
	[43010] = mage_FireWard_Entry,
	-- Frost Ward
	[6143] = mage_FrostWard_Entry,
	[8461] = mage_FrostWard_Entry,
	[8462] = mage_FrostWard_Entry,
	[10177] = mage_FrostWard_Entry,
	[28609] = mage_FrostWard_Entry,
	[32796] = mage_FrostWard_Entry,
	[43012] = mage_FrostWard_Entry,
	-- Ice Barrier
	[11426] = mage_IceBarrier_Entry,
	[13031] = mage_IceBarrier_Entry,
	[13032] = mage_IceBarrier_Entry,
	[13033] = mage_IceBarrier_Entry,
	[27134] = mage_IceBarrier_Entry,
	[33405] = mage_IceBarrier_Entry,
	[43038] = mage_IceBarrier_Entry,
	[43039] = mage_IceBarrier_Entry,			
	-- Mana Shield
	[1463] = mage_ElementalWard_Entry,
	[8494] = mage_ElementalWard_Entry,
	[8495] = mage_ElementalWard_Entry,
	[10191] = mage_ElementalWard_Entry,
	[10192] = mage_ElementalWard_Entry,
	[10193] = mage_ElementalWard_Entry,
	[27131] = mage_ElementalWard_Entry,
	[43019] = mage_ElementalWard_Entry,
	[43020] = mage_ElementalWard_Entry,		
	
	-- PALADIN
	-- Sacred Shield
	[58597] = {1.0, paladin_SacredShield_Create, generic_Hit},
	
	-- PRIEST	
	-- Power Word: Shield
	[17] 	= priest_PWS_Entry,
	[592]	= priest_PWS_Entry,
	[600] 	= priest_PWS_Entry,
	[3747] 	= priest_PWS_Entry,
	[6065] 	= priest_PWS_Entry,
	[10898] = priest_PWS_Entry,
	[10899] = priest_PWS_Entry,
	[10900] = priest_PWS_Entry,
	[10901] = priest_PWS_Entry,
	[25217] = priest_PWS_Entry,
	[25218] = priest_PWS_Entry,
	[48065] = priest_PWS_Entry,
	[48066] = priest_PWS_Entry,
	-- Divine Aegis
	[47753] = {1.0, priest_DivineAegis_Create, generic_Hit},
	
	-- ROGUE
	-- Cheat Death
	--   No clue how it works technically (does it still uses an absorb?)
	--   While there is no absorb value to watch for, it would screw our
	--   numbers on other effects.
	
	-- SHAMAN
	-- Glyph of Stoneclaw Totem
	--   This absorb effect cannot be supported without heavy exceptional
	--   handling, because it is neither applied by SPELL_AURA_APPLIED or
	--   nor removed by SPELL_AURA_REMOVED - but only an UNIT_AURA event is
	--   fired on gain/loss. The gain can actually be detected by the cast 
	--   of Stoneclaw Totem (and knowledge about the glyphs), but not if it
	--   breaks prematurely, making looping through all buffs needed everytime
	--	 it could be active and UNIT_AURA is fired. At least the amount would
	--	 be constant for every rank.
	-- The normal totem absorb effect could be tracked with more ease, but still
	-- there is no proper removal event, so it would rely on zombified absorb
	-- effects until the totem drops. Maybe later. (What about totem GUIDs?)
	-- [55277] = {1.0, shaman_GlyphStoneclawTotem_Create, generic_Hit},
	
	-- WARLOCK
	-- Sacrifice
	[7812] = warlock_Sacrifice_Entry,
	[19438] = warlock_Sacrifice_Entry,
	[19440] = warlock_Sacrifice_Entry,
	[19441] = warlock_Sacrifice_Entry,
	[19442] = warlock_Sacrifice_Entry,
	[19443] = warlock_Sacrifice_Entry,
	[27273] = warlock_Sacrifice_Entry,
	[47985] = warlock_Sacrifice_Entry,
	[47986] = warlock_Sacrifice_Entry,
	-- Shadow Ward
	[6229] = warlock_ShadowWard_Entry,
	[11739] = warlock_ShadowWard_Entry,
	[11740] = warlock_ShadowWard_Entry,
	[28610] = warlock_ShadowWard_Entry,
	[47890] = warlock_ShadowWard_Entry,
	[47891] = warlock_ShadowWard_Entry,

	-- ITEMS	
	-- Val'anyr
	[64413] = {1.0, items_Valanyr_Create, generic_Hit},
	-- Essence of Gossamer
	[60218] = {5.0, function() return 4000, 1.0; end, items_EssenceOfGossamer_Hit},
};


AbsorbsMonitor.Triggers = {	
	-- Val'anyr
	["64411_OnAuraApplied"] = items_Valanyr_OnAuraApplied,
	["64411_OnAuraRemoved"] = items_Valanyr_OnAuraRemoved,
};
