
-- we keep the Ace3 dependency to a minimum
local L = AceLibrary("AceLocale-2.2"):new("GridStatusAbsorbsMonitor")

L:RegisterTranslations("enUS", function()
	return {
		["Absorb effects left"] = true,
	}
end)

L:RegisterTranslations("deDE", function()
	return {
		["Absorb effects left"] = "Verbleibende Absorptionseffekte",
	}
end)


local GridStatusAbsorbsMonitor = Grid:GetModule("GridStatus"):NewModule("GridStatusAbsorbsMonitor");
GridStatusAbsorbsMonitor.menuName = "Absorb effects left";

-- AceEvent2 has no support for local addon messages
-- sent by AceEvent3
-- we cannot embed it though, since we could screw
-- the mixin's from grid
local AceEvent = LibStub("AceEvent-3.0");


GridStatusAbsorbsMonitor.defaultDB = {
	unitAbsorbsLeft = {
		enable = true,
		priority = 90,
		color = { r = 1, g = 0, b = 0, a = 1 },
	},
};

local GridStatusAM_profile;


-- To prevent table allocation on every update
local color = {r = 0, g = 0, b = 0, a = 0.5};


function GridStatusAbsorbsMonitor:OnInitialize()
	self.super.OnInitialize(self)
	self:RegisterStatus("unitAbsorbsLeft", "Absorb effects left", nil, true)	
	GridStatusAM_profile = self.db.profile.unitAbsorbsLeft;
end

function GridStatusAbsorbsMonitor:OnEnable()
    self:RegisterEvent("Grid_UnitJoined");
    
    AceEvent.RegisterMessage(GridStatusAbsorbsMonitor, "Absorbs_Unit_UpdatedTotal");
    AceEvent.RegisterMessage(GridStatusAbsorbsMonitor, "Absorbs_Unit_Cleared");
end

function GridStatusAbsorbsMonitor:OnDisable()
	AceEvent.UnregisterMessage(GridStatusAbsorbsMonitor, "Absorbs_Unit_UpdatedTotal");
	AceEvent.UnregisterMessage(GridStatusAbsorbsMonitor, "Absorbs_Unit_Cleared");
end

function GridStatusAbsorbsMonitor:Grid_UnitJoined(guid, unitId)
    self.core:SendStatusLost(guid, "unitAbsorbsLeft");
end

-- obj is the same as self, but it gets called by the . operator
-- within AceEvent3
function GridStatusAbsorbsMonitor:Absorbs_Unit_UpdatedTotal(obj, guid, value, quality)
	local shieldText;
	
	if(value > 999) then
		shieldText = string.format("%.1fk", value / 1000.0);
	else
		shieldText = string.format("%3.0f", value);
	end
	
	color.r = 1 - quality;
	color.g = quality;
	--color.a = 0.3 + (quality * 0.3);

	self.core:SendStatusGained(guid, "unitAbsorbsLeft", GridStatusAM_profile.priority, nil, color, shieldText)
end

function GridStatusAbsorbsMonitor:Absorbs_Unit_Cleared(obj, guid)
	self.core:SendStatusLost(guid, "unitAbsorbsLeft")
end
